#ifndef GNM_SF_BESSEL_H_
#define GNM_SF_BESSEL_H_

#include <numbers.h>

gnm_float gnm_bessel_i (gnm_float x, gnm_float alpha);
gnm_float gnm_bessel_j (gnm_float x, gnm_float alpha);
gnm_float gnm_bessel_k (gnm_float x, gnm_float alpha);
gnm_float gnm_bessel_y (gnm_float x, gnm_float alpha);

#endif
